import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import './media/exceptionWidget.css.js';
import { localizeWithPath } from 'vscode/vscode/vs/nls';
import { append, isAncestorOfActiveElement, $ as $$1 } from 'vscode/vscode/vs/base/browser/dom';
import { ZoneWidget } from 'vscode/vscode/vs/editor/contrib/zoneWidget/browser/zoneWidget';
import { EDITOR_CONTRIBUTION_ID } from 'vscode/vscode/vs/workbench/contrib/debug/common/debug';
import { RunOnceScheduler } from 'vscode/vscode/vs/base/common/async';
import { IThemeService } from 'vscode/vscode/vs/platform/theme/common/themeService';
import { ThemeIcon } from 'vscode/vscode/vs/base/common/themables';
import { registerColor } from 'vscode/vscode/vs/platform/theme/common/colorRegistry';
import { IInstantiationService } from 'vscode/vscode/vs/platform/instantiation/common/instantiation';
import { LinkDetector } from './linkDetector.js';
import { ActionBar } from 'vscode/vscode/vs/base/browser/ui/actionbar/actionbar';
import { Action } from 'vscode/vscode/vs/base/common/actions';
import { widgetClose } from 'vscode/vscode/vs/platform/theme/common/iconRegistry';

const $ = $$1;
const debugExceptionWidgetBorder = registerColor('debugExceptionWidget.border', { dark: '#a31515', light: '#a31515', hcDark: '#a31515', hcLight: '#a31515' }, ( localizeWithPath(
    'vs/workbench/contrib/debug/browser/exceptionWidget',
    'debugExceptionWidgetBorder',
    'Exception widget border color.'
)));
const debugExceptionWidgetBackground = registerColor('debugExceptionWidget.background', { dark: '#420b0d', light: '#f1dfde', hcDark: '#420b0d', hcLight: '#f1dfde' }, ( localizeWithPath(
    'vs/workbench/contrib/debug/browser/exceptionWidget',
    'debugExceptionWidgetBackground',
    'Exception widget background color.'
)));
let ExceptionWidget = class ExceptionWidget extends ZoneWidget {
    constructor(editor, exceptionInfo, debugSession, themeService, instantiationService) {
        super(editor, { showFrame: true, showArrow: true, isAccessible: true, frameWidth: 1, className: 'exception-widget-container' });
        this.exceptionInfo = exceptionInfo;
        this.debugSession = debugSession;
        this.instantiationService = instantiationService;
        this.applyTheme(themeService.getColorTheme());
        this._disposables.add(themeService.onDidColorThemeChange(this.applyTheme.bind(this)));
        this.create();
        const onDidLayoutChangeScheduler = ( new RunOnceScheduler(() => this._doLayout(undefined, undefined), 50));
        this._disposables.add(this.editor.onDidLayoutChange(() => onDidLayoutChangeScheduler.schedule()));
        this._disposables.add(onDidLayoutChangeScheduler);
    }
    applyTheme(theme) {
        this.backgroundColor = theme.getColor(debugExceptionWidgetBackground);
        const frameColor = theme.getColor(debugExceptionWidgetBorder);
        this.style({
            arrowColor: frameColor,
            frameColor: frameColor
        });
    }
    _applyStyles() {
        if (this.container) {
            this.container.style.backgroundColor = this.backgroundColor ? ( this.backgroundColor.toString()) : '';
        }
        super._applyStyles();
    }
    _fillContainer(container) {
        this.setCssClass('exception-widget');
        const fontInfo = this.editor.getOption(50 );
        container.style.fontSize = `${fontInfo.fontSize}px`;
        container.style.lineHeight = `${fontInfo.lineHeight}px`;
        container.tabIndex = 0;
        const title = $('.title');
        const label = $('.label');
        append(title, label);
        const actions = $('.actions');
        append(title, actions);
        label.textContent = this.exceptionInfo.id ? ( localizeWithPath(
            'vs/workbench/contrib/debug/browser/exceptionWidget',
            'exceptionThrownWithId',
            'Exception has occurred: {0}',
            this.exceptionInfo.id
        )) : ( localizeWithPath(
            'vs/workbench/contrib/debug/browser/exceptionWidget',
            'exceptionThrown',
            'Exception has occurred.'
        ));
        let ariaLabel = label.textContent;
        const actionBar = ( new ActionBar(actions));
        actionBar.push(( new Action('editor.closeExceptionWidget', ( localizeWithPath('vs/workbench/contrib/debug/browser/exceptionWidget', 'close', "Close")), ThemeIcon.asClassName(widgetClose), true, async () => {
            const contribution = this.editor.getContribution(EDITOR_CONTRIBUTION_ID);
            contribution?.closeExceptionWidget();
        })), { label: false, icon: true });
        append(container, title);
        if (this.exceptionInfo.description) {
            const description = $('.description');
            description.textContent = this.exceptionInfo.description;
            ariaLabel += ', ' + this.exceptionInfo.description;
            append(container, description);
        }
        if (this.exceptionInfo.details && this.exceptionInfo.details.stackTrace) {
            const stackTrace = $('.stack-trace');
            const linkDetector = this.instantiationService.createInstance(LinkDetector);
            const linkedStackTrace = linkDetector.linkify(this.exceptionInfo.details.stackTrace, true, this.debugSession ? this.debugSession.root : undefined);
            stackTrace.appendChild(linkedStackTrace);
            append(container, stackTrace);
            ariaLabel += ', ' + this.exceptionInfo.details.stackTrace;
        }
        container.setAttribute('aria-label', ariaLabel);
    }
    _doLayout(_heightInPixel, _widthInPixel) {
        this.container.style.height = 'initial';
        const lineHeight = this.editor.getOption(66 );
        const arrowHeight = Math.round(lineHeight / 3);
        const computedLinesNumber = Math.ceil((this.container.offsetHeight + arrowHeight) / lineHeight);
        this._relayout(computedLinesNumber);
    }
    focus() {
        this.container?.focus();
    }
    hasFocus() {
        if (!this.container) {
            return false;
        }
        return isAncestorOfActiveElement(this.container);
    }
};
ExceptionWidget = ( __decorate([
    ( __param(3, IThemeService)),
    ( __param(4, IInstantiationService))
], ExceptionWidget));

export { ExceptionWidget };
